/***************************************************************************
 *   Copyright Brian Ledbetter 2001-2003 <brian@shadowcom.net>             *
 *   Copyright Ravikiran Rajagopal 2003                                    *
 *   ravi@ee.eng.ohio-state.edu                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License (version 2) as   *
 *   published by the Free Software Foundation. (The original KSplash/ML   *
 *   codebase (upto version 0.95.3) is BSD-licensed.)                      *
 *                                                                         *
 ***************************************************************************/

#include <kapplication.h>
#include <kconfig.h>
#include <kdebug.h>
#include <kdialogbase.h>
#include <kgenericfactory.h>
#include <kglobalsettings.h>
#include <kiconloader.h>
#include <kprogress.h>
#include <ksplash/objkstheme.h>
#include <kstandarddirs.h>
#include <kwin.h>

#include <qlabel.h>
#include <qwidget.h>

#include "thememacx.h"
#include "thememacx.moc"

K_EXPORT_COMPONENT_FACTORY( libksplashmacx, KGenericFactory<ThemeMacX> );

ThemeMacX::ThemeMacX( QWidget *parent, const char *name, const QStringList &args )
    :ThemeEngine( parent, name, args )
{
  KWin::setState( winId(), NET::Modal | NET::StaysOnTop | NET::SkipTaskbar | NET::SkipPager );
  KWin::setType( winId(), NET::Dock );

  readSettings();
  initUi();
}

ThemeMacX::~ThemeMacX()
{
  delete mIconBar;
}

void ThemeMacX::slotSetPixmap( const QString& icon )
{
  QString s( locate( "data", mTheme->themeDir() + "/" + icon ) );
  s = s.isNull()?locate( "data", QString("Themes/")+mTheme->themeDir() + "/" + icon ):s;
  s = s.isNull()?locate( "appdata", mTheme->themeDir() + "/" + icon ):s;
  s = s.isNull()?locate( "appdata", QString("Themes/")+mTheme->themeDir() + "/" + icon ):s;

  if( !s.isNull() )
    mIconBar->appendIcon( new Icon( QPixmap( s ), mMinSize, mMaxSize, mOptimizedRendering ) );
  else
    mIconBar->appendIcon( new Icon( DesktopIcon( icon, mMaxSize ), mMinSize, mMaxSize, mOptimizedRendering ) );
}

void ThemeMacX::initUi()
{
  QLabel *lbl = new QLabel( this );
  lbl->show();
  QString splashWidget = locate( "data", mTheme->themeDir() + "/Splash.png" );
  splashWidget = splashWidget.isNull()?locate( "appdata", "pics/splash_top.png" ):splashWidget;

  if( splashWidget.isNull() )
  {
    resize( 0, 0 );
    move( 0, 0 );
  }
  else
  {
    QRect desk(KGlobalSettings::splashScreenDesktopGeometry());
    QPixmap px( splashWidget );
    lbl->setPixmap( px );
    resize( px.width(), px.height() );
    move( desk.width()/2 - px.width()/2, desk.height()/2 - px.height()/2 );
  }

  setBackgroundMode( NoBackground );
  lbl->setBackgroundMode( NoBackground );

  mProgress = new KProgress( lbl );
  if (!mProgressPos.isValid())
  {
    QSize s(lbl->sizeHint()-mProgress->sizeHint()-QSize(4,4));
    mProgressPos.setTopLeft( QPoint(s.width(), s.height()) );
    mProgressPos.setSize( mProgress->sizeHint() );
  }
  mProgress->resize( mProgressPos.size() );
  mProgress->move( mProgressPos.topLeft() );

  mIconBar = new IconBar;
  mIconBar->show();
}

void ThemeMacX::readSettings()
{
  if( !mTheme )
    return;

  KConfig *cfg = mTheme->themeConfig();
  if( !cfg )
    return;

  cfg->setGroup( QString("KSplash Theme: %1").arg(mTheme->theme()) );

  mMinSize = cfg->readNumEntry( "Icon Size Minimum", 16 );
  mMaxSize = cfg->readNumEntry( "Icon Size Maximum", 64 );
  mOptimizedRendering = cfg->readBoolEntry( "Optimized Icon Rendering", true );
  mUseProgress = cfg->readBoolEntry( "Progress Bar Visible", true );
  QRect defaultPos;
  mProgressPos = cfg->readRectEntry( "Progress Bar Position", &defaultPos );

  // not currently implemented.
  mIconsJumping = cfg->readBoolEntry( "Icons Jumping", false );
}
